unit RandomGenerator;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  MouseRNG, StdCtrls;

const
  BUFFER_SIZE = 1024*1024*8;  // 1MB of data


type
  TRandomGenerator_F = class(TForm)
    pbOK: TButton;
    pbCancel: TButton;
    MouseRNG: TMouseRNG;
    lblStored: TLabel;
    lblRequired: TLabel;
    lblInstructions: TLabel;
    procedure FormShow(Sender: TObject);
    procedure pbCancelClick(Sender: TObject);
    procedure pbOKClick(Sender: TObject);
    procedure MouseRNGByteGenerated(Sender: TObject; random: Byte);
    procedure FormResize(Sender: TObject);
  private
  protected
    FRequiredBits: integer;
    FBitsAlreadyGot: integer;

    FBitsCollected: integer;
    FData: array [1..BUFFER_SIZE] of byte;

    procedure UpdateDisplay();
  public
    property RequiredBits: integer write FRequiredBits;
    property BitsAlreadyGot: integer write FBitsAlreadyGot;
    
    property BitsCollected: integer read FBitsCollected;

    // Copy up to "count" bits into the buffer, return the number of bits copied
    // Note: "buffer" is indexed from 0
    function GetBitsCollected(count: integer; var buffer: array of byte): integer;

    procedure Wipe();

  end;

implementation

{$R *.DFM}

uses
  SDUGeneral,
  Math;  // Required for "min(...)"

  
procedure TRandomGenerator_F.FormShow(Sender: TObject);
begin
  UpdateDisplay();
  MouseRNG.Enabled := TRUE;

end;


procedure TRandomGenerator_F.UpdateDisplay();
var
  enoughGenerated: boolean;
begin
  if FRequiredBits>0 then
    begin
    lblRequired.visible := TRUE;

    lblRequired.caption := 'Bits required: '+inttostr(FRequiredBits);

    enoughGenerated := (FBitsCollected>=FRequiredBits);

    SDUEnableControl(pbOK, enoughGenerated);
    if (enoughGenerated) then
      begin
      MouseRNG.Enabled := FALSE;
      MouseRNG.Color := clBtnFace;
      SetFocusedControl(pbOK);
      end;

    end
  else
    begin
    lblRequired.visible := FALSE;

    SDUEnableControl(pbOK, TRUE);
    end;

  lblStored.caption := 'Bits generated: '+inttostr(FBitsAlreadyGot + FBitsCollected);

end;



procedure TRandomGenerator_F.Wipe();
var
  i: integer;
begin
  MouseRNG.Enabled := FALSE;

  for i:=low(FData) to high(FData) do
    begin
    FData[i] := 0;
    end;

  FRequiredBits:= 0;
  FBitsCollected:= 0;

end;



procedure TRandomGenerator_F.pbCancelClick(Sender: TObject);
begin
  MouseRNG.Enabled := FALSE;
  Wipe();
  ModalResult := mrCancel;

end;

procedure TRandomGenerator_F.pbOKClick(Sender: TObject);
begin
  MouseRNG.Enabled := FALSE;
  ModalResult := mrOK;
  
end;


// Copy up to "count" bits into the buffer, return the number of bits copied
function TRandomGenerator_F.GetBitsCollected(count: integer; var buffer: array of byte): integer;
var
  copyCountBytes: integer;
  i: integer;
begin
  copyCountBytes := min((count div 8), (FBitsCollected div 8));

  for i:=0 to (copyCountBytes-1) do
    begin
    buffer[i] := FData[(FBitsCollected div 8) - i];
    FData[(FBitsCollected div 8) - i] := 0;
    end;

  FBitsCollected := FBitsCollected - (copyCountBytes * 8);

  Result := copyCountBytes * 8;
end;

procedure TRandomGenerator_F.MouseRNGByteGenerated(Sender: TObject;
  random: Byte);
begin
  inc(FBitsCollected, 8);
  FData[(FBitsCollected div 8)] := random;
  UpdateDisplay();

end;

procedure TRandomGenerator_F.FormResize(Sender: TObject);
var
  twoButtonsFullWidth: integer;
  twoButtonsGap: integer;
begin
  // Horizontally center the labels...
  lblInstructions.left := (self.width-lblInstructions.width) div 2;
  lblStored.left := (self.width-lblStored.width) div 2;
  lblRequired.left := (self.width-lblRequired.width) div 2;

  // Nicely horizontally center the buttons...
  twoButtonsGap := pbCancel.left - (pbOK.left + pbOK.width);
  twoButtonsFullWidth := pbOK.width + twoButtonsGap + pbCancel.width;

  pbOK.left := (self.width-twoButtonsFullWidth) div 2;
  pbCancel.left := pbOK.left + pbOK.width + twoButtonsGap;


end;

END.


