unit SDLargeDivision;


// NOTE: Although this software only allows division/comparison by up to 256
//       (a byte is passed in), there is no reason why this can't be changed
//       to a WORD to limit it to division by up to 65536 by:
//          1) Changing all instances of "byte" to "WORD"
//          2) Changing all instances of "word" to "DWORD"
//       To increase even more:
//          1) Changing all instances of "byte" to "DWORD"
//          2) Changing all instances of "word" to "int64"
//        - this will allow division by up to 4294967296



interface

const
  MAX_BYTES = 1024;  // The largest number that this software can store is
                     // MAX_BYTES * 8 (the size of a byte)
  

type
  TSDLargeNum = array [0..MAX_BYTES] of byte;


procedure SD_Large_zero(var x: TSDLargeNum);
procedure SD_Large_copy(src: TSDLargeNum; var dest: TSDLargeNum);
function  SD_Large_compare(x: TSDLargeNum; y: byte): integer;
procedure SD_Large_divide(x: TSDLargeNum; y: byte; var Q: TSDLargeNum; var R: byte);


implementation

uses
  sysutils,  // Required for Exception
  Windows;  // Required for DWORD



// Set the specified variable to zero
procedure SD_Large_zero(var x: TSDLargeNum);
var
  i: integer;
begin
  for i:=high(x) downto low(x) do
    begin
    x[i] := 0;
    end;

end;


procedure SD_Large_copy(src: TSDLargeNum; var dest: TSDLargeNum);
var
  i: integer;
begin
  for i:=high(src) downto low(src) do
    begin
    dest[i] := src[i];
    end;

end;


// Compare a large number with another number
// Returns;
//   x = y: return 0
//   x > y: return 1
//   x < y: return -1
function SD_Large_compare(x: TSDLargeNum; y: byte): integer;
var
  i: integer;
  retval: integer;
begin
  retval := 0;

  // Scan through the array, apart from the last byte
  // If any of these array elements are set to >0, then the large number must
  // be larger than y (which is only byte)
  for i:=high(TSDLargeNum) downto (low(TSDLargeNum)+1) do
    begin
    if (x[i]>0) then
      begin
      retval := 1;
      break;
      end;
    end;

  if (retval = 0) then
    begin
    if (x[0]<y) then
      begin
      retval := -1;
      end;
    end;

  Result := retval;
end;


// Divide a large number by y; returning the integer division in Q, and the
// remainder in R
// WARNING: No divide by zero checking is carried out, if "y" is set to zero
procedure SD_Large_divide(x: TSDLargeNum; y: byte; var Q: TSDLargeNum; var R: byte);
var
  tmp: WORD;  // The type of this variable **MUST** be capable of holding
              // ((2 x the largest value that "y" can be set to) - 1) 
  i: integer;
begin
  // Reject silly...
  if (y=0) then
    begin
    raise Exception.Create('Cannot do large division divide by zero');
    end;

  tmp := 0;
  // This is pretty crude; it should skip all the high bytes until it reaches
  // an array element which is non-zero
  for i:=high(x) downto low(x) do
    begin
    tmp := (tmp shl 8) + x[i];

    Q[i] := tmp div y;
    R := tmp - (Q[i] * y);
    tmp := R;
    end;

end;


END.


